<?php

if (!defined('ABSPATH')) {
    exit;
}
require_once MYFATOORAH_WOO_PLUGIN_PATH . 'includes/libraries/MyfatoorahLoader.php';
require_once MYFATOORAH_WOO_PLUGIN_PATH . 'includes/libraries/MyfatoorahLibrary.php';

if (!class_exists('WC_Gateway_Myfatoorah')) {
    include_once('class-wc-gateway-myfatoorah.php');
}

/**
 * Myfatoorah_V2 Payment Gateway class.
 *
 * Extended by individual payment gateways to handle payments.
 *
 * @class       WC_Gateway_Myfatoorah_pg
 * @extends     WC_Payment_Gateway
 */
class WC_Gateway_Myfatoorah_v2 extends WC_Gateway_Myfatoorah {

    protected $code;
    protected $count           = 0;
    protected $gateways        = [];
    protected $appleRegistered = false;
    protected $totalAmount     = 0;
    protected $myfatoorah;

    /**
     * Constructor
     */
    public function __construct() {
        $this->code               = 'v2';
        $this->method_description = __("MyFatoorah Debit/Credit Card payment.", 'myfatoorah-woocommerce');
        $this->method_title       = __('MyFatoorah - Cards', 'myfatoorah-woocommerce');
        parent::__construct();

        $this->has_fields = true;
    }

//-----------------------------------------------------------------------------------------------------------------------------------------

    /**
     * Process the payment and return the result.
     * 
     * @param int $orderId
     * @return array
     */
    public function process_payment($orderId) {

        $curlData = $this->getPayLoadData($orderId);

        $gateway = myfatoorahInputData('mf_gateway', INPUT_POST) ?? 'myfatoorah';

        $configId = myfatoorahInputData('mfData', INPUT_POST);
        $data     = $this->mf->getInvoiceURL($curlData, $gateway, $orderId, $configId);
        update_post_meta($orderId, 'InvoiceId', $data['invoiceId']);

        return array(
            'result'   => 'success',
            'redirect' => $data['invoiceURL'],
        );
    }

//-----------------------------------------------------------------------------------------------------------------------------------------

    function payment_fields_v2() {

        if (!empty($this->mfError)) {
            include(MYFATOORAH_WOO_PLUGIN_PATH . 'templates/error.php');
        } else if (isset($this->newDesign) && $this->newDesign == 'yes') {

            if ($this->listOptions === 'multigateways') {
                include_once(MYFATOORAH_WOO_PLUGIN_PATH . 'templates/form.php');
            }
            include_once(MYFATOORAH_WOO_PLUGIN_PATH . 'templates/paymentFields.php');
        } else {
            include_once(MYFATOORAH_WOO_PLUGIN_PATH . 'templates/paymentFieldsV2.php');
        }
    }

//-----------------------------------------------------------------------------------------------------------------------------------------

    /**
     * Return the gateway's title.
     *
     * @return string
     */
    public function get_title() {
        $this->setGateways();
        if ($this->listOptions === 'multigateways' && $this->count == 1 && $this->isSingleView === 'yes') {
            // option new design 
            if (isset($this->newDesign) && $this->newDesign == 'yes') {
                return ($this->lang == 'ar') ? $this->gateways['all'][0]->PaymentMethodAr : $this->gateways['all'][0]->PaymentMethodEn;
            } else {
                return ($this->lang == 'ar') ? $this->gateways[0]->PaymentMethodAr : $this->gateways[0]->PaymentMethodEn;
            }
        } else {
            return apply_filters('woocommerce_gateway_title', $this->title, $this->id);
        }
    }

//-----------------------------------------------------------------------------------------------------------------------------------------

    /**
     * Return the gateway's icon.
     *
     * @return string
     */
    public function get_icon() {

        if ($this->listOptions === 'multigateways' && $this->count == 1 && $this->isSingleView === 'yes') {
            if (isset($this->newDesign) && $this->newDesign == 'yes') {
                $logo = $this->gateways['all'][0]->ImageUrl;
            } else {
                $logo = $this->gateways[0]->ImageUrl;
            }
            $icon = '<img src="' . $logo . '" alt="' . esc_attr($this->get_title()) . '" style="margin: 0px; width: 50px; height: 30px;"/>';
        } else {
            $icon = $this->icon ? '<img src="' . WC_HTTPS::force_https_url($this->icon) . '" alt="' . esc_attr($this->get_title()) . '" />' : '';
        }

        return apply_filters('woocommerce_gateway_icon', $icon, $this->id);
    }

//-----------------------------------------------------------------------------------------------------------------------------------------
    protected function setGateways() {
        if ($this->listOptions === 'myfatoorah' || count($this->gateways) != 0) {
            return;
        }

        $httpReferer = myfatoorahInputData('HTTP_REFERER', INPUT_SERVER);
        if (!is_ajax() || !isset($httpReferer) || stripos($httpReferer, get_permalink(wc_get_page_id('checkout'))) !== 0) {
            return;
        }

        // option new design 
        try {
            if (isset($this->newDesign) && $this->newDesign == 'yes') {
//                $totals = WC()->cart->total;
//                $this->gateways = $this->mf->getPaymentMethodsForDisplay($totals, get_woocommerce_currency());

                $totals                = WC()->cart->get_totals();
                $this->totalAmount     = $totals['total'];
                $this->appleRegistered = ($this->registerApplePay == 'yes') ? true : false;
                $this->gateways = $this->mf->getPaymentMethodsForDisplay($this->totalAmount, get_woocommerce_currency(), $this->appleRegistered);
                $this->count    = count($this->gateways['all']);
            } else {
                $gateways = $this->mf->getCachedPaymentMethods();

                $embedOptions = get_option('woocommerce_myfatoorah_embedded_settings');

                $this->gateways = (isset($embedOptions) && $embedOptions['enabled'] == 'yes') ? $gateways['cards'] : $gateways['all'];
                $this->count    = count($this->gateways);
            }
        } catch (Exception $ex) {
            $this->mfError = $ex->getMessage();
        }
    }

//-----------------------------------------------------------------------------------------------------------------------------------------

    /**
     * Don't enable this payment, if there is no API key
     * 
     * @param type $key
     * @param type $value
     * 
     * @return string
     */
    public function validate_enabled_field($key, $value) {
        $enabled = is_null($value) ? 'no' : 'yes';

        if ($enabled == 'yes') {

            //don't enable if there is no API key
            $apiKey = $this->get_field_value('apiKey', $this->form_fields['apiKey']);
            if (!$apiKey) {
                WC_Admin_Settings::add_error(__('You should add the API key first, to enable this payment method', 'myfatoorah-woocommerce'));
                $enabled = 'no';
            } else {
                $countryMode      = $this->get_field_value('countryMode', $this->form_fields['countryMode']);
                $testMode         = $this->get_field_value('testMode', $this->form_fields['testMode']);
                $this->myfatoorah = new PaymentMyfatoorahApiV2($apiKey, $countryMode, ($testMode === 'yes'), $this->pluginlog);
                try {
                    $this->myfatoorah->getVendorGateways();
                } catch (Exception $ex) {
                    WC_Admin_Settings::add_error(__($ex->getMessage(), 'myfatoorah-woocommerce'));
                    $enabled = 'no';
                }
            }
        }

        return $enabled;
    }

    //-----------------------------------------------------------------------------------------------------------------------------------------

    /**
     * Don't disable this invoiceItem, if shipping method is enabled
     * 
     * @param type $key
     * @param type $value
     * 
     * @return string
     */
    public function validate_invoiceItems_field($key, $value) {
        $enabled         = is_null($value) ? 'no' : 'yes';
        $shippingOptions = get_option('woocommerce_myfatoorah_shipping_settings');

        if ($enabled == 'no' && isset($shippingOptions['enabled']) && $shippingOptions['enabled'] == 'yes') {
            WC_Admin_Settings::add_error(__('You can not disable invoice items option while MyFatoorah Shipping is enabled', 'myfatoorah-woocommerce'));
            $enabled = 'yes';
        }

        return $enabled;
    }

//-----------------------------------------------------------------------------------------------------------------------------------------

    /**
     * Disable the embedded if the new design enabled
     * 
     * @param type $key
     * @param type $value
     * 
     * @return string
     */
    public function validate_newDesign_field($key, $value) {
        $active = is_null($value) ? 'no' : 'yes';

        if ($active == 'yes') {
            $embedOptions = get_option('woocommerce_myfatoorah_embedded_settings');

            $enableFieldValue = myfatoorahInputData($this->get_field_key('enabled'), INPUT_POST); //get_field_value without validation to avoid duplicate error message
            $apiKey           = $this->get_field_value('apiKey', $this->form_fields['apiKey']); //don't disable if there is no API key
            if ($apiKey && $enableFieldValue && isset($embedOptions['enabled']) && $embedOptions['enabled'] == 'yes') {
                $embedOptions['enabled'] = 'no';
                update_option('woocommerce_myfatoorah_embedded_settings', apply_filters('woocommerce_settings_api_sanitized_fields_' . 'myfatoorah_embedded', $embedOptions), 'yes');
            }
        }

        return $active;
    }

    //-----------------------------------------------------------------------------------------------------------------------------------------

    /**
     * Keep register Apple Pay value
     * 
     * @param type $key
     * @param type $value
     * 
     * @return string
     */
    public function validate_registerApplePay_field($key, $value) {
        $active = is_null($value) ? 'no' : 'yes';
        if ($active == 'no' || $this->get_field_value('enabled', $this->form_fields['enabled']) == 'no')
            return $active;


        if ($this->get_field_value('listOptions', $this->form_fields['listOptions']) == 'myfatoorah' ||
                $this->get_field_value('newDesign', $this->form_fields['newDesign']) == 'no'
        ) {
            $active = 'no';
            WC_Admin_Settings::add_error(__('Please make sure to select New design and List all gateway option to enable Apple Pay Embedded.', 'myfatoorah-woocommerce'));
            return $active;
        }

        try {
            $data       = $this->myfatoorah->registerApplePayDomain(get_site_url());
            if ($data->Message == 'OK') {
                return 'yes';
            }
            WC_Admin_Settings::add_error($data->Message);
            return 'no';
        } catch (Exception $ex) {
            WC_Admin_Settings::add_error($ex->getMessage());
            return 'no';
        }
    }

//-----------------------------------------------------------------------------------------------------------------------------------------

    /**
     * Return whether or not this gateway still requires setup to function.
     *
     * When this gateway is toggled on via AJAX, if this returns true a
     * redirect will occur to the settings page instead.
     * @since 3.4.0
     *
     * @return bool
     */
    public function needs_setup() {

        if (empty($this->apiKey)) {
            return true;
        }

        $embedOptions = get_option('woocommerce_myfatoorah_embedded_settings');
        if (isset($this->newDesign) && $this->newDesign == 'yes' && isset($embedOptions['enabled']) && $embedOptions['enabled'] == 'yes') {
            return true;
        }
        return false;
    }

//-----------------------------------------------------------------------------------------------------------------------------------------
}
